import requests
import logging


class Psps:
    def check_info_psp(idPsp: int):
        """
        Check info psps in moises by id.

        Params:
            idPsp (int): psps ids in moises  

        Return:
            success (boolean): success function
            message (string): result of function
            payload (dict) :
                active (int)
                endDate(str)
                idCrmPspCat (int)
                insertDate (int)
                name (str)
                visibility (int)
                settings (dict):
                    available (bool)
                    maxAmount (int)
                    mintAmount (int)
                    selectCountries (list[str])
                    availableCountries (list(int))
                    notAvailableBusinessUnit (list(int))

        """
        url = "http://webservicesnt.org:5050/get/info-psps"

        payload = {}
        payload['idPsp'] = str(idPsp)

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('RESPONE CHECK INFO PSPS %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone = {}
                respone['success'] = False
                respone['message'] = "psps not found"
                respone['payload'] = {}

                return respone

            respone = {}
            respone['success'] = True
            respone['message'] = "psp info correctly"
            respone['payload'] = moisesResponse['data']
            return respone
    
        except Exception as Err:
            respone = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone
    
    def activate_psp(idPsp: int):
        """
        Activate the psp and set 'active' in both the visibility and active fields.

        Params:
            idPsp (int): psps ids in moises  

        Return:
            result (int): success function
            error (string): result of function
            data (dict) : result message

        """
        
        url = "http://webservicesnt.org:5050/update/cat-psp-active"
    
        
        payload = {}
        payload['idPsp'] = idPsp
        payload['active'] = 1

        headers = {}
        headers['Content-Type'] = 'application/json'
        
        try:

            response = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('response active cat psp'.format(response.text))
            response = response.json()

            if not response['result'] == 1:
                response_psp = {}
                response_psp['success'] = False
                response_psp['message'] = "psp not updated"
                response_psp['payload'] = {}

                return response_psp

            response_psp = {}
            response_psp['success'] = True
            response_psp['message'] = "psp updated correctly"
            response_psp['payload'] = response['data']
            return response_psp
    
        except Exception as Err:
            response_psp = {}
            response_psp['success'] = False
            response_psp['message'] = str(Err)
            response_psp['payload'] = {}
            
            return response_psp


from marshmallow import Schema, fields, ValidationError

class get_info_psp_settings(Schema):
    available = fields.Boolean(required=True, description="true if all correctly")
    maxAmount = fields.Integer(required=True, description="true if all correctly")
    minAmount = fields.Integer(required=True, description="true if all correctly")
    selectedCountries = fields.List(required=True, description="true if all correctly")
    availableCountries = fields.List(required=True, description="true if all correctly")
    availableBusinessUnits = fields.List(required=True, description="true if all correctly")
    notAvailableBusinessUnit = fields.List(required=True, description="true if all correctly")

    class Meta:
        ordered = True

class get_info_psp_schema(Schema):
    idCrmPspCat = fields.Integer(required=True, description="true if all correctly")
    name = fields.String(required=True, description="true if all correctly")
    settings = fields.Nested(get_info_psp_settings, required=False)

    class Meta:
        ordered = True

class get_info_psps(Schema):
    success = fields.Boolean(required=True, description="true if all correctly")
    message = fields.String(required=True, description="lastname person to assigns transaction")
    payload = fields.Nested(get_info_psp_schema, required=False)
    
    class Meta:
        ordered = True

class get_info_psps_output:
    def create(body: get_info_psps):
        try:
            return get_info_psps().load(body)
        except ValidationError as err:
            raise Exception(err)


from marshmallow import Schema, fields, ValidationError

class check_email_phone_input_schema(Schema):
    email = fields.Email(required=True, description="Email lead")
    phone = fields.String(required=True, description = "Phone email")
    
    class Meta:
        ordered = True

class check_email_phone_input:
    def create(body: check_email_phone_input_schema):
        try:
            return check_email_phone_input_schema().load(body)
        except ValidationError as err:
            raise Exception(err)


from marshmallow import Schema, fields, ValidationError

class customer_schema(Schema):
    tpid = fields.String(required=True, description="true if all correctly")
    crmId = fields.String(required=True, description="true if all correctly")
    firstName = fields.String(required=True, description="true if all correctly")
    lastName = fields.String(required=True, description="true if all correctly")
    phoneCode = fields.String(required=True, description="true if all correctly")
    phoneNumber = fields.String(required=True, description="true if all correctly")
    country = fields.String(required=True, description="true if all correctly")
    BussinessUnit = fields.String(required=True, description="true if all correctly")
    class Meta:
        ordered = True

class get_customer_output_schema(Schema):
    success = fields.Boolean(required=True, description="true if all correctly")
    message = fields.String(required=True, description="lastname person to assigns transaction")
    payload = fields.Nested(customer_schema, required=False)
    
    class Meta:
        ordered = True

class get_customer_output:
    def create(body: get_customer_output_schema):
        try:
            return get_customer_output_schema().load(body)
        except ValidationError as err:
            raise Exception(err)


import requests
import logging

from .dto import (
    create_transaction_input_schema,
    transaction_output_schema
)

class Transaction:
    def create_transaction(request: create_transaction_input_schema) -> transaction_output_schema:
        """
        Create transaction in moises.

        Params:
            create_transaction_input_schema : 
                idPsp (string): id from psp
                crmId (string): lead's phone
                amount (Int) 

        Return:
            transaction_output_schema:
                success (boolean): success function
                message (string): result of function
        """

        url = "http://crm-app.tech/api/crm/create-transaction"

        payload = {}
        payload['idPsp'] = request['idPsp']
        payload['crmId'] = request['crmId']
        payload['amount'] = request['amount']

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('CREATE TRANSACTION IN MOISES %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : transaction_output_schema = {}
                respone['success'] = False
                respone['message'] = "Transaction not create"

                return respone

            respone : transaction_output_schema = {}
            respone['success'] = True
            respone['message'] = "Transaction create"
            respone['payload'] = {}
            return respone
    
        except Exception as Err:
            respone : transaction_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone
        
    
    def create_cashout(amount: int) -> transaction_output_schema:
        """
        Create transaction cashout in moises.

        Params:
            amount (Int) 

        Return:
            transaction_output_schema:
                success (boolean): success function
                message (string): result of function
                
                payload (dict) : 
                    idTransaction [str] : id of transaction
        """

        url = "http://webservicesnt.org:5050/create-cashout"

        payload = {}
        payload['amount'] = amount

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('CREATE TRANSACTION CASHOUT IN MOISES %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : transaction_output_schema = {}
                respone['success'] = False
                respone['message'] = "Transaction not create"

                return respone

            respone : transaction_output_schema = {}
            respone['success'] = True
            respone['message'] = "Transaction create"
            respone['payload'] = {}
            respone['payload']['idTransaction'] = moisesResponse['data']['id']
            return respone
    
        except Exception as Err:
            respone : transaction_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone
        
    
    def desactive_cashout(amount: str) -> transaction_output_schema:
        """
        Desactive transaction cashout in moises.

        Params:
            idTransaction (str) 

        Return:
            transaction_output_schema:
                success (boolean): success function
                message (string): result of function
        """

        url = "http://webservicesnt.org:5050/cancel-cashouts"

        payload = {}
        payload['amount'] = amount

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('CANCEL TRANSACTION CASHOUT IN MOISES %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : transaction_output_schema = {}
                respone['success'] = False
                respone['message'] = "Transaction not desactivate"

                return respone

            respone : transaction_output_schema = {}
            respone['success'] = True
            respone['message'] = "Transaction desactivate"
            respone['payload'] = {}
            return respone
    
        except Exception as Err:
            respone : transaction_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone


import requests
import logging

from .dto import (
    check_email_phone_input_schema,
    get_customer_output_schema,
    move_lead_output_schema, 
    create_lead_input_schema,
    create_customer_output_schema
)

class Customers:
    def check_customer_by_email_phone(request: check_email_phone_input_schema) -> get_customer_output_schema:
        """
        Check if exist a customer in moises whit the email and phone.

        Params:
            check_email_phone_input_schema : 
                email (string): lead's email
                phone (string): lead's phone

        Return:
            get_customer_output_schema:
                success (boolean): success function
                message (string): result of function
                payload (dict) -- OPTIONAL:
                    tpid (string): if customer exist return tpid
        """
        url = "http://webservicesnt.org:5050/check-if-exist-customer"

        payload = {}
        payload['email'] = request['email']
        payload['phone'] = request['phone']

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('RESPONE CHECK IF EXIST IN MOISES %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : get_customer_output_schema = {}
                respone['success'] = False
                respone['message'] = "Customer exist"
                respone['payload'] = {}
                respone['payload']['tpid'] = moisesResponse['data'][0]['tpId']

                return respone

            respone : get_customer_output_schema = {}
            respone['success'] = True
            respone['message'] = "Customer not exist"
            respone['payload'] = {}
            return respone
    
        except Exception as Err:
            respone : get_customer_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone
        
    def get_info_by_tpid(tpid: str):
        """
        Get info of customer by tpid.

        Params:
            tpid (string) :  lead's tpid

        Return:
            get_customer_output_schema:
                success (boolean): success function
                message (string): result of function
                payload (dict) -- OPTIONAL:
                    tpid (string): if customer exist return tpid
        """
        url = "http://webservicesnt.org:5050/customer/info"

        payload = {}
        payload['tpid'] = tpid

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('RESPONE GET INFO FROM CUSTOMER %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : get_customer_output_schema = {}
                respone['success'] = False
                respone['message'] = "Customer noit exist"
                respone['payload'] = {}

                return respone

            respone : get_customer_output_schema = {}
            respone['success'] = True
            respone['message'] = "Customer info"
            respone['payload'] = {}
            respone['payload']['tpid'] = moisesResponse['data']['tpId']
            respone['payload']['crmId'] = moisesResponse['data']['crmId']
            respone['payload']['firstName'] = moisesResponse['data']['firstName']
            respone['payload']['lastName'] = moisesResponse['data']['lastName']
            respone['payload']['phoneCode'] = moisesResponse['data']['phoneCode']
            respone['payload']['phoneNumber'] = moisesResponse['data']['phoneNumber']
            respone['payload']['country'] = moisesResponse['data']['country']
            respone['payload']['BussinessUnit'] = moisesResponse['data']['idCrmBusinessUnit']

            return respone
    
        except Exception as Err:
            respone : get_customer_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone
   
    def move_lead_another_bussinesUnit(tpid: str, bussinesUnitName: str) -> move_lead_output_schema:
        """
        move a customer in moises to another bussinesUnit and change owner in crm-app.

        Params:
            tpid (string): lead's TPID
            bussinesUnitName (str): BussinesUnit to move
                Fxtrategy     : 8
                AscendingBull : 14
                Inverlion     : 15
                Noimarkets    : 16
                UBM Capital   : 17
                ALL Markets   : 18
                FxIntegral    : 19
                Profitbitz    : 20
                Solutraders   : 21
                BearInvester  : 22

        Return:
            move_lead_output_schema:
                success (boolean): success function
                message (string): result of function
                payload (dict) -- OPTIONAL:
                    tpid (string): if customer move correctly return tpid
                    crmId (string): if customer move correctly return crmId
                    idBusinessUnit (string): if customer move correctly return idBusinessUnit
        """
        url = "http://webservicesnt.org:5050/update/move-lead-crm"

        payload = {}
        payload['tpid'] = tpid
        payload['bussinesUnitName'] = bussinesUnitName

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('MOVE LEAD IN MOISES %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : move_lead_output_schema = {}
                respone['success'] = False
                respone['message'] = "Customer not move"
                respone['payload'] = {}

                return respone

            respone : move_lead_output_schema = {}
            respone['success'] = True
            respone['message'] = "Customer move correctly"
            respone['payload'] = {}
            respone['payload']['tpid'] = tpid
            respone['payload']['crmId'] = moisesResponse['data']['crmId']
            respone['payload']['idBusinessUnit'] = moisesResponse['data']['idBussinesUnit']

            return respone
        
        except Exception as Err:
            respone : get_customer_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone
    
    def create_lead_affiliates(input: create_lead_input_schema) -> create_customer_output_schema:
        """
        create a lead from api affiliates in a internal Affiliate.

        Params:
            create_lead_input_schema: 
                firstName (str) : lead's name
                lastName (str) : lead's last name
                email (str) : lead's email
                password (str) :  lead's password
                phoneCode (int) : country phone code
                phoneNumber (str) : lead's phone number
                country (str) : lead's country
                    MX, PE, CL, BR, CR, PA

                page (str) : lead's page

                bussinesUnitName (str): lead's BussinesUnit
                    Fxtrategy     : 8
                    AscendingBull : 14
                    Inverlion     : 15
                    Noimarkets    : 16
                    UbmCapital    : 17
                    AllMarkets    : 18
                    FxIntegral    : 19
                    Profitbitz    : 20
                    Solutraders   : 21
                    BearInvester  : 22

        Return:
            create_customer_output_schema:
                success (boolean): success function
                message (string): result of function
                payload (dict) -- OPTIONAL:
                    id (string): if customer move correctly return UUID
                  
        """
        url = "https://webservicesnt.org:4455/customer/affiliate-api"

        payload = {}
        payload['firstName'] = input['firstName']
        payload['lastName'] = input['firstName']
        payload['email'] = input['email']
        payload['password'] = input['password']
        payload['phoneCode'] = int(input['phoneCode'])
        payload['phoneNumber'] = input['phoneNumber']
        payload['country'] = input['country']
        payload['page'] = input['page']
        payload['brand'] = input['bussinesUnitName']

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('CREATE LEAD IN API AFFILIATES MOISES %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone : create_customer_output_schema = {}
                respone['success'] = False
                respone['message'] = "Customer not create"
                respone['payload'] = {}

                return respone

            respone : create_customer_output_schema = {}
            respone['success'] = True
            respone['message'] = "Customer move correctly"
            respone['payload'] = {}
            respone['payload'] = moisesResponse['data']

            return respone
        
        except Exception as Err:
            respone : create_customer_output_schema = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone



import requests
import logging

class BusinessUnit:
    def check_info_BU(id_business_unit: int):
        """
        Check info BU in moises by id.

        Params:
            id_business_unit (int): bu ids in moises  

        Return:
            success (boolean): success function
            message (string): result of function
            payload (dict) :
                name (string)


        """
        url = "http://webservicesnt.org:5050/get/bussinesUnit"

        payload = {}
        payload['idBussinessUnit'] = str(id_business_unit)

        headers = {}
        headers['Content-Type'] = 'application/json'

        try:

            moisesResponse = requests.request("POST", url, headers=headers, json=payload)
            logging.warning('RESPONE CHECK INFO BUSSINESS UNIT %s' % (moisesResponse.text))
            moisesResponse = moisesResponse.json()

            if not moisesResponse['result'] == 1:
                respone = {}
                respone['success'] = False
                respone['message'] = "BU not found"
                respone['payload'] = {}

                return respone

            respone = {}
            respone['success'] = True
            respone['message'] = "BU info correctly"
            respone['payload'] = {}
            respone['payload']['id'] = id_business_unit
            respone['payload']['name'] = str(moisesResponse['data']['name']).lower()
            return respone
    
        except Exception as Err:
            respone = {}
            respone['success'] = False
            respone['message'] = str(Err)
            respone['payload'] = {}

            return respone


from setuptools import setup

setup(
    name="pymoises",
    version="1.0.8",
    description="Callings from moises endpoints...",
    author="Platimex",
    packages=[
        "pymoises", 
        "pymoises.dto", 
        "pymoises.dto.input", 
        "pymoises.dto.input.customer",
        "pymoises.dto.input.transaction",
        "pymoises.dto.output",
        "pymoises.dto.output.customer",
        "pymoises.dto.output.transaction",
        "pymoises.dto.output.psps"
        ],
    install_requires=['requests', 'marshmallow'],
)


from marshmallow import Schema, fields, ValidationError

class create_transaction_input_schema(Schema):
    idPsp = fields.String(required=True)
    crmId = fields.String(required=True)
    amount = fields.Integer(required=True)
    
    class Meta:
        ordered = True

class create_transaction_input:
    def create(body: create_transaction_input_schema):
        try:
            return create_transaction_input_schema().load(body)
        except ValidationError as err:
            raise Exception(err)


from marshmallow import Schema, fields, ValidationError

class transaction_output_schema(Schema):
    success = fields.Boolean(required=True, description="true if all correctly")
    message = fields.String(required=True, description="lastname person to assigns transaction")
    
    class Meta:
        ordered = True

class transaction_output:
    def create(body: transaction_output_schema):
        try:
            return transaction_output_schema().load(body)
        except ValidationError as err:
            raise Exception(err)


from marshmallow import Schema, fields, ValidationError, validate
from marshmallow.validate import Length, Regexp

bussinesUnitName = [
    'Profitbitz', 
    'FxIntegral', 
    'AscendingBull', 
    'Inverlion', 
    'Noimarkets', 
    'Solutraders', 
    'BearInvester', 
    'Fxtrategy', 
    'AllMarkets', 
    'UbmCapital'
]

countries = ["MX", "PE", "CL", "BR", "CR", "PA"]

class create_lead_input_schema(Schema):
    firstName = fields.String(required=True, description = "Phone email")
    lastName = fields.String(required=True, description = "Phone email")
    email = fields.Email(required=True, description="Email lead")
    password = fields.String(validate=[Regexp(regex=('^(?=\w*\d)(?=\w*[A-Z])(?=\w*[a-z])\S{8,15}$'))])      
    phoneCode = fields.Integer(required=True, description = "Phone email")
    phoneNumber = fields.String(required=True, description = "Phone email")
    country = fields.String(
        required=True, 
        validate=validate.OneOf(countries), 
        description = "Phone email")        
    page = fields.String(required=True, description = "Phone email")
    bussinesUnitName = fields.String(
        required=True, 
        validate=validate.OneOf(bussinesUnitName), 
        description="Bussines Unit name", 
    )    
    class Meta:
        ordered = True

class create_lead_input:
    def create(body: create_lead_input_schema):
        try:
            return create_lead_input_schema().load(body)
        except ValidationError as err:
            raise Exception(err)